/** animator.dart --- Manager of Animation instances.
 *
 * Copyright (C) 2014  Miguel Guedes
 * 
 * Author: Miguel Guedes <miguel.a.guedes@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see http://www.gnu.org/licenses/.
 *
 * Comments:
 * 
 */


import 'dart:html';

import 'animation.dart';
import 'animation_exception.dart';


class Animator
{
  static final Map<Element, Map<String, Animation> > _animations
    = new Map<Element, Map<String, Animation> >();


  static Animation
  create(final Element  element,
         final String   property,
       [ final Duration duration,
         final String   easing ] )
  {
    final Animation animation = new Animation(
        element,
        property,
        duration,
        easing);

    return animation.start();
  }
  
  static List<Animation>
  createAll(final Element       element,
            final List<String>  properties,
          [ final Duration      duration,
            final String        easing ] )
  {
    final List<Animation> list = new List<Animations>();
      
    for(final String property in properties) {
      final Animation animation = new Animation(
          element,
          property,
          duration,
          easing);

      animation.start();
      list.add(animation);
    }

    return list;
  }

  static void add(final Animation animation)
  {
    final Element element  = animation.getElement();
    final String  property = animation.getProperty();
    
    if(!animation.running())
      throw new AnimationException("Animation not running");
    
    if(_animations.containsKey(element)
        && _animations[element].containsKey(property)) {
      _animations[element][property].stop();
    } else if(_animations[element] == null)
      _animations[element] = new Map<String, Animation>();
    
    _animations[element][property] = animation;
    
  }

  static void remove(final Animation animation)
  {
    final Element element  = animation.getElement();
    final String  property = animation.getProperty();
    
    if(animation.running())
      throw new AnimationException("Animation running");
    else if(!_animations.containsKey(element)
        || !_animations[element].containsKey(property)) {
      throw new AnimationException("Animation not contained");
    }
    
    final Map<Element, Map<String, Animation> > m = _animations[element];

    m.remove(property);
    
    if(m.length == 0)
      _animations.remove(element);
  }
}