/** easing.dart --- Easing functions.
 *
 * Copyright (C) 2014  Miguel Guedes
 * 
 * Author: Miguel Guedes <miguel.a.guedes@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see http://www.gnu.org/licenses/.
 *
 * Comments:
 *
 * Uses the awesome easing formulas by Robert Penner [0]. His terms of use are
 * included in the comment block below.
 * 
 * I didn't actually write the functions defined below in the class Easing.  I
 * simply packaged them up as static methods in a class and cleaned up the code
 * slightly.  The core logic remains unaltered, however.
 * 
 * [0] http://www.robertpenner.com/easing/
 * 
 */

/*
 * TERMS OF USE - EASING EQUATIONS
 * 
 * Open source under the BSD License. 
 * 
 * Copyright © 2001 Robert Penner
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.  Redistributions in binary
 * form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 
 * Neither the name of the author nor the names of contributors may be used to
 * endorse or promote products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */


class Easing
{
	/* t: current time, b: start value, c: end value, d: total time */
	static double easeInQuad(double t, double b, double c, double d)
    => c*(t/=d)*t + b;
  
	static double easeOutQuad(double t, double b, double c, double d)
    => -c *(t/=d)*(t-2) + b;
  
	static double easeInOutQuad(double t, double b, double c, double d)
  {
		if((t/=d/2) < 1)
      return c/2*t*t + b;
    
		return -c/2 * ((--t)*(t-2) - 1) + b;
	}
  
	static double easeInCubic(double t, double b, double c, double d)
    => c*(t/=d)*t*t + b;
  
	static double easeOutCubic(double t, double b, double c, double d)
    => c*((t=t/d-1)*t*t + 1) + b;
  
	static double easeInOutCubic(double t, double b, double c, double d)
  {
		if((t/=d/2) < 1)
      return c/2*t*t*t + b;
    
		return c/2*((t-=2)*t*t + 2) + b;
	}
  
	static double easeInQuart(double t, double b, double c, double d)
    => c*(t/=d)*t*t*t + b;
  
	static double easeOutQuart(double t, double b, double c, double d)
    => -c * ((t=t/d-1)*t*t*t - 1) + b;
  
	static double easeInOutQuart(double t, double b, double c, double d)
  {
		if((t/=d/2) < 1)
      return c/2*t*t*t*t + b;
    
		return -c/2 * ((t-=2)*t*t*t - 2) + b;
	}
  
	static double easeInQuint(double t, double b, double c, double d)
    => c*(t/=d)*t*t*t*t + b;
  
	static double easeOutQuint(double t, double b, double c, double d)
    => c*((t=t/d-1)*t*t*t*t + 1) + b;
  
	static double easeInOutQuint(double t, double b, double c, double d)
  {
		if((t/=d/2) < 1)
      return c/2*t*t*t*t*t + b;
    
		return c/2*((t-=2)*t*t*t*t + 2) + b;
	}
    
	static double easeInSine(double t, double b, double c, double d)
    => -c * Math.cos(t/d * (Math.PI/2)) + c + b;
  
	static double easeOutSine(double t, double b, double c, double d)
    => c * Math.sin(t/d * (Math.PI/2)) + b;
  
	static double easeInOutSine(double t, double b, double c, double d)
    => -c/2 * (Math.cos(Math.PI*t/d) - 1) + b;
    
	static double easeInExpo(double t, double b, double c, double d)
    => (t==0) ? b : c * Math.pow(2, 10 * (t/d - 1)) + b;
  
	static double easeOutExpo(double t, double b, double c, double d)
    => (t==d) ? b+c : c * (-Math.pow(2, -10 * t/d) + 1) + b;
  
	static double easeInOutExpo(double t, double b, double c, double d)
  {
		if(t==0)
      return b;
    
		if(t==d)
      return b+c;
    
		if((t/=d/2) < 1)
      return c/2 * Math.pow(2, 10 * (t - 1)) + b;
    
		return c/2 * (-Math.pow(2, -10 * --t) + 2) + b;
	}
  
	static double easeInCirc(double t, double b, double c, double d)
    => -c * (Math.sqrt(1 - (t/=d)*t) - 1) + b;
  
	static double easeOutCirc(double t, double b, double c, double d)
    =>c * Math.sqrt(1 - (t=t/d-1)*t) + b;
  
	static double easeInOutCirc(double t, double b, double c, double d)
  {
		if((t/=d/2) < 1)
      return -c/2 * (Math.sqrt(1 - t*t) - 1) + b;
    
		return c/2 * (Math.sqrt(1 - (t-=2)*t) + 1) + b;
	}
  
	static double easeInElastic(double t, double b, double c, double d)
  {
		double s = 1.70158;
    double p = 0;
    double a = c;
    
		if(t == 0)
      return b;

    if((t /= d) == 1)
      return b + c;

    if(p == 0)
      p = d*.3;

    double s;
    
		if(a < Math.abs(c)) {
      a = c;
      s = p/4;
    } else
      s = p/(2*Math.PI) * Math.asin (c/a);
    
		return -(a*Math.pow(2,10*(t-=1)) * Math.sin( (t*d-s)*(2*Math.PI)/p )) + b;
	}
  
	static double easeOutElastic(double t, double b, double c, double d)
  {
		double s = 1.70158;
    double p = 0;
    double a = c;
    
		if(t == 0)
      return b;

    if((t /= d) == 1)
      return b + c;

    if(p == 0)
      p = d*.3;

    double s;
		if(a < Math.abs(c)) {
      a = c;
      s = p / 4;
    } else
      s = p / (2 * Math.PI) * Math.asin (c / a);
    
		return a*Math.pow(2,-10*t) * Math.sin( (t*d-s)*(2*Math.PI)/p ) + c + b;
	}
  
	static double easeInOutElastic(double t, double b, double c, double d)
  {
		double s = 1.70158;
    double p = 0;
    double a = c;
    
		if(t == 0)
      return b;

    if((t /= d / 2) ==2 )
      return b + c;

    if(p == 0)
      p = d * (.3 * 1.5);

    double s;
		if(a < Math.abs(c)) {
      a = c;
      s= p / 4;
    } else
      s = p / (2 * Math.PI) * Math.asin(c / a);
    
		if(t < 1)
      return -.5*(a*Math.pow(2,10*(t-=1)) * Math.sin( (t*d-s)*(2*Math.PI)/p )) + b;
    
		return a*Math.pow(2,-10*(t-=1)) * Math.sin( (t*d-s)*(2*Math.PI)/p )*.5 + c + b;
	}
  
	static double easeInBack(double t, double b, double c, double d, [double s])
  {
		if (s == null)
      s = 1.70158;
    
		return c*(t/=d)*t*((s+1)*t - s) + b;
	}
  
	static double easeOutBack(double t, double b, double c, double d, [double s])
  {
		if(s == null)
      s = 1.70158;
    
		return c*((t=t/d-1)*t*((s+1)*t + s) + 1) + b;
	}
  
	static double easeInOutBack(double t, double b, double c, double d, [double s])
  {
		if(s == null)
      s = 1.70158;
    
		if((t /= d / 2) < 1)
      return c/2*(t*t*(((s*=(1.525))+1)*t - s)) + b;
    
		return c/2*((t-=2)*t*(((s*=(1.525))+1)*t + s) + 2) + b;
	}
  
	static double easeInBounce(double t, double b, double c, double d)
    => c - easeOutBounce (d - t, 0, c, d) + b;
    
	static double easeOutBounce(double t, double b, double c, double d)
  {
		if ((t /= d) < (1 / 2.75))
			return c * (7.5625 * t * t) + b;
		else if(t < (2 / 2.75))
			return c * (7.5625 * (t -= (1.5 / 2.75)) * t + .75) + b;
		else if(t < (2.5 / 2.75))
			return c * (7.5625 * (t -= (2.25 / 2.75)) * t + .9375) + b;
		else
			return c * (7.5625 * (t -= (2.625 / 2.75)) * t + .984375) + b;
	}
  
	static double easeInOutBounce(double t, double b, double c, double d)
  {
		if(t < d/2)
      return easeInBounce(t * 2, 0, c, d) * .5 + b;
    
		return easeOutBounce(t * 2 - d, 0, c, d) * .5 + c * .5 + b;
	}
}
